@@ -6,6 +6,8 @@ from queue import Queue,Empty
 import logging
 import time
 import uuid
+from datetime import datetime, timedelta
+import pytz
 #import pdb
 
 
@@ -16,8 +18,6 @@ class TradeClass():
         self.tsocket=None
         self.tsessionKey=""
         self.teventQueue=Queue()
-        self.datas=[]
-        self.last = ""
         self.LoginOK=False
         self.m_objZMQKeepAlive = None
         self.orderinfo={}
@@ -66,42 +66,42 @@ class TradeClass():
         return data
 
     #查询当日委托回报
-    def QryReport(self,  qryIndex=""):
-        if  qryIndex=="":
-            self.datas=[]
-            self.last=""
-        self.lock.acquire()
-        obj = {"Request":"RESTOREREPORT","SessionKey":self.tsessionKey,"QryIndex":qryIndex}
-        self.tsocket.send_string(json.dumps(obj))
-        message = self.tsocket.recv()[:-1]
-        data = json.loads(message)
-        self.lock.release()
-        if data["Reply"] == "RESTOREREPORT":
-            if len(data["Orders"]) == 0:
-                return self.datas
-            for ord in data["Orders"]:
-                self.datas.append(ord)
-                self.last = ord["QryIndex"]
-            return self.QryReport(self.last)
+    def QryReport(self):
+        datas=[]
+        last=""
+        while True:
+            self.lock.acquire()
+            obj = {"Request":"RESTOREREPORT","SessionKey":self.tsessionKey,"QryIndex":last}
+            self.tsocket.send_string(json.dumps(obj))
+            message = self.tsocket.recv()[:-1]
+            data = json.loads(message)
+            self.lock.release()
+            if data["Reply"] == "RESTOREREPORT":
+                if len(data["Orders"]) == 0:
+                    break
+                for ord in data["Orders"]:
+                    datas.append(ord)
+                    last = ord["QryIndex"]
+            return datas
 
     #查询当日成交回报
-    def QryFillReport(self, qryIndex=""):
-        if  qryIndex=="":
-            self.datas=[]
-            self.last=""
-        self.lock.acquire()
-        obj = {"Request":"RESTOREFILLREPORT","SessionKey":self.tsessionKey,"QryIndex":qryIndex}
-        self.tsocket.send_string(json.dumps(obj))
-        message = self.tsocket.recv()[:-1]
-        data = json.loads(message)
-        self.lock.release()
-        if data["Reply"] == "RESTOREFILLREPORT":
-            if len(data["Orders"]) == 0:
-                return self.datas
-            for ord in data["Orders"]:
-                self.datas.append(ord)
-                self.last = ord["QryIndex"]
-            return self.QryFillReport(self.last)
+    def QryFillReport(self):
+        datas=[]
+        last=""
+        while True:
+            self.lock.acquire()
+            obj = {"Request":"RESTOREFILLREPORT","SessionKey":self.tsessionKey,"QryIndex":last}
+            self.tsocket.send_string(json.dumps(obj))
+            message = self.tsocket.recv()[:-1]
+            data = json.loads(message)
+            self.lock.release()
+            if data["Reply"] == "RESTOREFILLREPORT":
+                if len(data["Orders"]) == 0:
+                    break
+                for ord in data["Orders"]:
+                    datas.append(ord)
+                    last = ord["QryIndex"]
+            return datas
     
     #获取委托单信息
     def getorderinfo(self, ordid):
@@ -155,7 +155,16 @@ class TradeClass():
         elif data['ErrCode']=="-25":
             print("价格不能小于和等于 0 (市价类型不会去检查) ")
             return
-
+    #策略单
+    def NewStrategyOrder(self, param):
+        self.lock.acquire()
+        obj = {"Request":"NEWSTRATEGYORDER","SessionKey":self.tsessionKey}
+        obj["Param"] = param
+        self.tsocket.send_string(json.dumps(obj))
+        message = self.tsocket.recv()[:-1]
+        data = json.loads(message)
+        self.lock.release()
+        return data
 
     #改单
     def ReplaceOrder(self, param):
@@ -193,41 +202,41 @@ class TradeClass():
         return data
 
     #查询持仓
-    def QryPosition(self,accountMask,qryIndex=""):
-        if  qryIndex=="":
-            self.datas=[]
-            self.last=""
-        self.lock.acquire()
-        obj = {"Request":"POSITIONS","SessionKey":self.tsessionKey,"AccountMask":accountMask,"QryIndex":qryIndex}
-        self.tsocket.send_string(json.dumps(obj))
-        message = self.tsocket.recv()[:-1]
-        data = json.loads(message)
-        self.lock.release()
-        if data["Reply"] == "POSITIONS":
-            if len(data["Positions"]) == 0:
-                return self.datas
-            for ord in data["Positions"]:
-                self.datas.append(ord)
-                self.last = ord["QryIndex"]
-            return self.QryPosition(accountMask,self.last)
+    def QryPosition(self,accountMask):
+        datas=[]
+        last=""
+        while True:
+            self.lock.acquire()
+            obj = {"Request":"POSITIONS","SessionKey":self.tsessionKey,"AccountMask":accountMask,"QryIndex":last}
+            self.tsocket.send_string(json.dumps(obj))
+            message = self.tsocket.recv()[:-1]
+            data = json.loads(message)
+            self.lock.release()
+            if data["Reply"] == "POSITIONS":
+                if len(data["Positions"]) == 0:
+                    break
+                for ord in data["Positions"]:
+                    datas.append(ord)
+                    last = ord["QryIndex"]
+            return datas
     #查询新建组合的委托回报
     def QryOptCombOrder(self, AM, qryIndex=""):
-        if  qryIndex=="":
-            self.datas=[]
-            self.last=""
-        self.lock.acquire()
-        obj = {"Request":"QUERYOPTCOMBORDER","SessionKey":self.tsessionKey,"AccountMask":AM,"QryIndex":qryIndex}
-        self.tsocket.send_string(json.dumps(obj))
-        message = self.tsocket.recv()[:-1]
-        data = json.loads(message)
-        self.lock.release()
-        if data["Reply"] == "QUERYOPTCOMBORDER":
-            if len(data['Orders']) == 0:
-                return self.datas
-            for ord in data['Orders']:
-                self.datas.append(ord)
-                self.last = ord["QryIndex"]
-            return self.QryOptCombOrder(AM, self.last)
+        datas=[]
+        last=""
+        while True:
+            self.lock.acquire()
+            obj = {"Request":"QUERYOPTCOMBORDER","SessionKey":self.tsessionKey,"AccountMask":AM,"QryIndex":last}
+            self.tsocket.send_string(json.dumps(obj))
+            message = self.tsocket.recv()[:-1]
+            data = json.loads(message)
+            self.lock.release()
+            if data["Reply"] == "QUERYOPTCOMBORDER":
+                if len(data['Orders']) == 0:
+                    break
+                for ord in data['Orders']:
+                    datas.append(ord)
+                    last = ord["QryIndex"]
+            return datas
     #建组合
     def OptComb(self, Param):
         self.lock.acquire()
@@ -238,6 +247,25 @@ class TradeClass():
         data = json.loads(message)
         self.lock.release()
         return data
+    #查询组合持仓
+    def QryOptCombPosition(self,accountMask, qryIndex=""):
+        datas=[]
+        last=""
+        while True:
+            self.lock.acquire()
+            obj = {"Request":"QUERYOPTCOMBPOSITION","SessionKey":self.tsessionKey,"AccountMask":accountMask,"QryIndex":last}
+            self.tsocket.send_string(json.dumps(obj))
+            message = self.tsocket.recv()[:-1]
+            data = json.loads(message)
+            self.lock.release()
+            if data["Reply"] == "QUERYOPTCOMBPOSITION":
+                if len(data["Positions"]) == 0:
+                    break
+                for ord in data["Positions"]:
+                    datas.append(ord)
+                    last = ord["QryIndex"]
+            return datas
+
 
     #查询持仓监控
     def QryPositionTracker(self):
@@ -283,9 +311,12 @@ class QuoteClass():
 
         self.eventQueue=Queue()
         self.lasthistorydata={}
-        self.event=threading.Event()
         self.m_objZMQKeepAlive = None
         self.LoginOK = False
+        self.realdate={}
+        self.status=[]
+
+        self.gmt = pytz.timezone('Etc/GMT')
 
     #def __del__(self):
     #    self.quote_logout()
@@ -329,6 +360,7 @@ class QuoteClass():
 
     #订阅实时报价
     def SubQuote(self,symbol):
+
         result=[]
         if not isinstance(symbol, list):
             symbol=[symbol]
@@ -414,25 +446,18 @@ class QuoteClass():
         # DOGS分K："DOGSK"
     #4: 历史数据开始时间,
     #5: 历史数据结束时间
-    def SubHistory(self,symbol, type, startTime, endTime):
+    def SubHistory(self,symbol, bartype, startTime, endTime,dbtype=""):
         if not (re.match("TC.S.", symbol) or re.match("TC.F.", symbol) or re.match("TC.O.", symbol) or\
         re.match("TC.SP.", symbol) or re.match("TC.F2.", symbol) or re.match("TC.O2.", symbol)):
             print("合约代码格式错误")
             return        
-        if type=="3K":
+        if bartype=="3K" or bartype=="5K" or bartype=="15K":
             m_type="1K"
-            duration="3"
-        elif type=="5K":
-            m_type="1K"
-            duration="5"
-        elif type=="15K":
-            m_type="1K"
-            duration="15"
         else:
-            m_type=type
-            duration="1"
-        if symbol+type not in self.lasthistorydata.keys() and int(int(endTime)/100)>=int(time.strftime("%Y%m%d",time.localtime())):
-            self.lasthistorydata[symbol+type]= {"Symbol": symbol,"DataType":type,"StartTime" :startTime,"EndTime" :endTime,"lastindex":""}
+            m_type=bartype
+
+        if symbol+bartype not in self.lasthistorydata.keys():#and int(int(endTime)/100)>=int(time.strftime("%Y%m%d",time.localtime())):
+            self.lasthistorydata[symbol+bartype]= {"Symbol": symbol,"DataType":bartype,"StartTime" :startTime,"EndTime" :endTime,"lastindex":""}
         self.lock.acquire()
         obj = {"Request":"SUBQUOTE","SessionKey":self.qsessionKey}
         obj["Param"] = {"Symbol": symbol,"SubDataType":m_type,"StartTime" :startTime,"EndTime" :endTime}
@@ -441,56 +466,52 @@ class QuoteClass():
         message = self.qsocket.recv()[:-1]
         data = json.loads(message)
         self.lock.release()
+        
+
         if data['Success']=='OK':
-            self.event.wait()
-            self.event.clear()
-
-        historydata=[]
-        def GetHistory(symbol, type, startTime, endTime, qryIndex=""):
-            self.lock.acquire()
-            obj = {"Request":"GETHISDATA","SessionKey":self.qsessionKey}
-            obj["Param"] = {"Symbol": symbol,"SubDataType":m_type,"Duration":duration,"StartTime" :startTime,"EndTime" :endTime,"QryIndex" :qryIndex}
-            self.qsocket.send_string(json.dumps(obj))
-            message = (self.qsocket.recv()[:-1]).decode("utf-8")
-            self.lock.release()
-            index =  re.search("{",message).span()[0] # filter
-            message = message[index:]
-            message = json.loads(message)
-
-            #print(message)
-            if len(message["HisData"]) == 0:
-                temp=historydata.copy()
-                historydata.clear()
-                if len(temp)!=0:
-                    if (type!="DOGSK" and type!="DOGSS"):
-                        if int(temp[-1]["Date"])<int(time.strftime("%Y%m%d",time.localtime())):
-                            self.UnsubHistory(symbol, type, startTime, endTime)
+            while True:
+                if self.getstatus(symbol, bartype, startTime, endTime,dbtype)=='Ready':
+                    temp=[]
+                    last=""
+                    if dbtype=="ODBC":
+                        if bartype=="DOGSK" or bartype=="DOGSS":
+                            temp.append('Ready')
+                            return temp
                         else:
-                            self.lasthistorydata[symbol+type]["lastindex"]=str(int(temp[-1]["QryIndex"])-1)
-                    else:
-                        if int(temp[-1]["d"])<int(time.strftime("%Y%m%d",time.localtime())):
-                            self.UnsubHistory(symbol, type, startTime, endTime)
+                            print("只有DOGSK和DOGSS周期支持ODBC")
+                            return None
+                    #his=self.gethistory(symbol, type, startTime, endTime)
+
+                    while True:
+                        his=self.__get_history(symbol, bartype, startTime, endTime,last)
+                        
+                        if his and len(his["HisData"])>0:
+                            for his1 in his["HisData"]:
+                                last=his1['QryIndex']
+                                temphis={"Symbol":symbol}
+                                if bartype=="DOGSK" or bartype=="DOGSS":
+                                    t=str(int(int(his1['t'])/1000))
+                                    temphis["DateTime"]=datetime.strptime(his1['d']+(" 0" if len(t)==5 else " ")+t,"%Y%m%d %H%M%S").replace(tzinfo=self.gmt)
+                                    #del his1["d"]
+                                    #del his1["t"]
+                                    #del his1["QryIndex"]
+                            
+                                elif bartype=="TICKS":
+                                    temphis["DateTime"]=datetime.strptime(his1['Date']+(" 0" if len(his1['FilledTime'])==5 else " ")+his1['FilledTime'],"%Y%m%d %H%M%S").replace(tzinfo=self.gmt)
+                                elif bartype=="1K" or bartype=="3K" or bartype=="5K" or bartype=="15K":
+                                    temphis["DateTime"]=datetime.strptime(his1['Date']+(" 0" if len(his1['Time'])==5 else " ")+his1['Time'],"%Y%m%d %H%M%S").replace(tzinfo=self.gmt)
+                                else:
+                                    temphis["DateTime"]=datetime.strptime(his1['Date']+" 070000","%Y%m%d %H%M%S").replace(tzinfo=self.gmt)
+                                #cnt = pytz.timezone('Asia/Shanghai')
+                                #temphis["DateTime"]=temphis["DateTime"].astimezone(tz=cnt)
+                                temphis.update(his1)
+                                temp.append(temphis)
                         else:
-                            self.lasthistorydata[symbol+type]["lastindex"]=str(int(temp[-1]["QryIndex"])-1)
-                return temp
-            for hisdata in message["HisData"]:
+                            self.status.remove({'DataType': m_type, 'StartTime':startTime, 'EndTime':endTime,'Symbol': symbol, 'Status': 'Ready'})
+                            self.UnsubHistory(symbol, bartype, startTime, endTime)
+                            break
+                    return temp
 
-                if 'd' in hisdata.keys():
-                    histemp={"Date":hisdata["d"],"Time":hisdata["t"]}
-                    histemp.update(hisdata)
-                    hisdata=histemp
-                if 'FilledTime' in hisdata.keys():
-                    histemp={"Date":hisdata["Date"],"Time":hisdata["FilledTime"]}
-                    del hisdata["Date"]
-                    histemp.update(hisdata)
-                    hisdata=histemp
-                #histemp={"Symbol":symbol}
-                #histemp.update(hisdata)
-                #hisdata=histemp
-                historydata.append(hisdata)
-                self.last = hisdata["QryIndex"]
-            return GetHistory(symbol, type, startTime, endTime,self.last)
-        return GetHistory(symbol, type, startTime, endTime)
 
     #解订历史数据
     #1：SessionKey，
@@ -522,39 +543,124 @@ class QuoteClass():
         if symbol+type in self.lasthistorydata.keys():
             del self.lasthistorydata[symbol+type]
         return data
- 
-    #分页获取订阅的历史数据，接口内部调用，无需外部调用，获取完整历史数据请使用SubHistory
-    # def GetHistory(self,symbol, type, startTime, endTime, qryIndex=""):
-    #     if not (re.match("TC.S.", symbol) or re.match("TC.F.", symbol) or re.match("TC.O.", symbol) or\
-    #     re.match("TC.SP.", symbol) or re.match("TC.F2.", symbol) or re.match("TC.O2.", symbol)):
-    #         print("合约代码格式错误")
-    #         return
-    #     if type=="3K":
-    #         m_type="1K"
-    #         duration="3"
-    #     elif type=="5K":
-    #         m_type="1K"
-    #         duration="5"
-    #     elif type=="15K":
-    #         m_type="1K"
-    #         duration="15"
-    #     else:
-    #         m_type=type
-    #         duration="1"
-    #     temp=[]
-    #     self.lock.acquire()
-    #     obj = {"Request":"GETHISDATA","SessionKey":self.qsessionKey}
-    #     obj["Param"] = {"Symbol": symbol,"SubDataType":m_type,"Duration":duration,"StartTime" :startTime,"EndTime" :endTime,"QryIndex" :qryIndex}
-    #     self.qsocket.send_string(json.dumps(obj))
-    #     message = (self.qsocket.recv()[:-1]).decode("utf-8")
-    #     self.lock.release()
-    #     index =  re.search("{",message).span()[0]  # filter
-    #     message = message[index:]
-    #     message = json.loads(message)
-    #     return message["HisData"]
+
+    def getstatus(self,symbol, bartype, startTime, endTime,dbtype):
+        #st={}
+        if bartype=="3K" or bartype=="5K" or bartype=="15K":
+            m_type="1K"
+        else:
+            m_type=bartype
+        st={'DataType': m_type, 'StartTime':startTime, 'EndTime':endTime,'Symbol': symbol, 'Status': 'Ready'}
+        if self.status and st in self.status:
+            return 'Ready'
+        else:
+            return ""
+
+    def gethistorymulti(self,symbol, type, startTime, endTime,dbtype=""):
+        th=[]
+        result=[]
+        if not isinstance(symbol, list):
+            symbol=[symbol]
+        for symb in symbol:
+            histhread=self.GethisThread(self.__gethis,symb, type, startTime, endTime,dbtype)
+            th.append(histhread)
+            histhread.start()
+        for th1 in th:
+            result.append(th1.get_result())
+        return result
+    def __gethis(self,symbol, bartype, startTime, endTime='time.strftime("%Y%m%d",time.localtime())+"07"',dbtype=""):
+        self.__subHistory(symbol, bartype, startTime, endTime,dbtype)
+        if bartype=="3K" or bartype=="5K" or bartype=="15K":
+            m_type="1K"
+        else:
+            m_type=bartype
+        while True:
+            if self.getstatus(symbol, bartype, startTime, endTime,dbtype)=='Ready':
+                temp=[]
+                if dbtype=="ODBC":
+                    if bartype=="DOGSK" or bartype=="DOGSS":
+                        # 使用 cursor() 方法创建一个游标对象 cursor
+                        temp.append('Ready')
+                        return temp
+                    else:
+                        print("只有DOGSK和DOGSS周期支持ODBC")
+                        return None
+                #his=self.gethistory(symbol, bartype, startTime, endTime)
+                last=""
+
+                while True:
+                    his=self.__get_history(symbol, bartype, startTime, endTime,last)
+                    if his and len(his["HisData"])>0:
+                        last=his["HisData"][-1]['QryIndex']
+                        temp=temp+his["HisData"]
+                    else:
+                        self.status.remove({'DataType': m_type, 'StartTime':startTime, 'EndTime':endTime,'Symbol': symbol, 'Status': 'Ready'})
+                        self.UnsubHistory(symbol, bartype, startTime, endTime)
+                        break
+                return temp
+    def __subHistory(self,symbol, type, startTime, endTime='time.strftime("%Y%m%d",time.localtime())+"07"',dbtype=""):
+        if not (re.match("TC.S.", symbol) or re.match("TC.F.", symbol) or re.match("TC.O.", symbol) or\
+        re.match("TC.SP.", symbol) or re.match("TC.F2.", symbol) or re.match("TC.O2.", symbol)):
+            print("合约代码格式错误")
+            return        
+        if type=="3K" or type=="5K" or type=="15K":
+            m_type="1K"
+        else:
+            m_type=type
+
+        obj = { "Request":"SUBQUOTE",
+                "SessionKey":self.qsessionKey,
+                "Param":{   "Symbol": symbol,
+                            "SubDataType":m_type,
+                            "StartTime" :startTime,
+                            "EndTime" :endTime,
+                            "DBType":dbtype
+                        }
+                }
+        self.lock.acquire()
+        self.qsocket.send_string(json.dumps(obj))
+        message = self.qsocket.recv()[:-1]
+        self.lock.release()
+        data = json.loads(message)
+        return data
+    #分页获取订阅的历史数据
+    def __get_history(self,symbol, type, startTime, endTime, qryIndex=""):
+        if type=="3K":
+            m_type="1K"
+            duration="3"
+        elif type=="5K":
+            m_type="1K"
+            duration="5"
+        elif type=="15K":
+            m_type="1K"
+            duration="15"
+        else:
+            m_type=type
+            duration="1"
+        obj = {"Request":"GETHISDATA",
+            "SessionKey":self.qsessionKey,
+            "Param":{"Symbol": symbol,
+                    "SubDataType":m_type,
+                    "Duration":duration,
+                    "StartTime" :startTime,
+                    "EndTime" :endTime,
+                    "QryIndex" :qryIndex
+                    }
+                }
+
+        self.lock.acquire()
+        self.qsocket.send_string(json.dumps(obj))
+        message = self.qsocket.recv()[:-1].decode("utf-8")
+        self.lock.release()
+        index =  re.search(":",message).span()[1]  # filter
+        message = message[index:]
+        #print(message)
+        message = json.loads(message)
+        return message
+
        
     #最新的K线数据
-    def barupdate(self,symbol, type):
+    def barupdate(self,symbol, bartype):
         if not (re.match("TC.S.", symbol) or re.match("TC.F.", symbol) or re.match("TC.O.", symbol) or\
         re.match("TC.SP.", symbol) or re.match("TC.F2.", symbol) or re.match("TC.O2.", symbol)):
             print("合约代码格式错误")
@@ -562,17 +668,17 @@ class QuoteClass():
         if symbol+type in self.lasthistorydata.keys():
             lastobj=self.lasthistorydata[symbol+type]
 
-            if type=="3K":
+            if bartype=="3K":
                 m_type="1K"
                 duration="3"
-            elif type=="5K":
+            elif bartype=="5K":
                 m_type="1K"
                 duration="5"
-            elif type=="15K":
+            elif bartype=="15K":
                 m_type="1K"
                 duration="15"
             else:
-                m_type=type
+                m_type=bartype
                 duration="1"
             temp=[]
             self.lock.acquire()
@@ -587,6 +693,96 @@ class QuoteClass():
             return message["HisData"]
         else:
             return
+
+    #最新的K线数据
+    def barupdate2(self,bartype,hisdata,realdata):
+        if realdata['DataType']=='REALTIME' and realdata['Quote']["Symbol"]==hisdata[-1]['Symbol'] and  (int(realdata['Quote']['FilledTime'])+80000)<int(realdata['Quote']['CloseTime']):
+            realdata['Quote']["DateTime"]=datetime.strptime(realdata['Quote']['TradeDate']+(" 0" if len(realdata['Quote']['FilledTime'])==5 else " ")+realdata['Quote']['FilledTime'],"%Y%m%d %H%M%S").replace(tzinfo=self.gmt)
+            #print(realdata['Quote']["DateTime"])
+            if bartype=="DK":
+                if realdata['Quote']["DateTime"].date()==hisdata[-1]["DateTime"].date():
+                    hisdata[-1]['Open']=realdata['Quote']["OpeningPrice"]
+                    hisdata[-1]['High']=realdata['Quote']["HighPrice"]
+                    hisdata[-1]['Low']=realdata['Quote']["LowPrice"]
+                    hisdata[-1]['Close']= realdata['Quote']["TradingPrice"]
+                    hisdata[-1]['Volume']= realdata['Quote']["TradeVolume"]
+                elif realdata['Quote']["DateTime"].date()>hisdata[-1]["DateTime"].date():
+                    temp={}
+                    
+                    temp['Symbol']=realdata['Quote']["Symbol"]
+                    temp['DateTime']=realdata['Quote']["DateTime"]
+                    temp['Date']=realdata['Quote']['TradeDate']
+                    temp['Time']=realdata['Quote']["FilledTime"]
+                    temp['UpTick']="0"
+                    temp['UpVolume']="0"
+                    temp['DownTick']="0"
+                    temp['DownVolume']="0"
+                    temp['UnchVolume']="0"
+
+                    temp['Open']=realdata['Quote']["OpeningPrice"]
+                    temp['High']=realdata['Quote']["HighPrice"]
+                    temp['Low']=realdata['Quote']["LowPrice"]
+                    temp['Close']= realdata['Quote']["TradingPrice"]
+                    temp['Volume']= realdata['Quote']["TradeVolume"]
+                    temp['QryIndex']= str(int(hisdata[-1]['QryIndex'])+1)
+
+                    hisdata.append(temp)
+            elif bartype=="1K" or bartype=="3K" or bartype=="5K" or bartype=="15K":
+                duration=float(bartype.strip("K"))
+                if hisdata[-1]["DateTime"]<=realdata['Quote']["DateTime"]:
+                    temp={}
+                    
+                    temp['Symbol']=realdata['Quote']["Symbol"]
+                    temp['DateTime']=hisdata[-1]["DateTime"]+timedelta(minutes=duration)
+                    temp['Date']=hisdata[-1]["Date"]
+                    temp['Time']=str(temp['DateTime'].hour*10000+temp['DateTime'].minute*100+temp['DateTime'].second)
+                    temp['UpTick']="0"
+                    temp['UpVolume']="0"
+                    temp['DownTick']="0"
+                    temp['DownVolume']="0"
+                    temp['UnchVolume']="0"
+
+                    temp['Open']=realdata['Quote']["TradingPrice"]
+                    temp['High']=realdata['Quote']["TradingPrice"]
+                    temp['Low']=realdata['Quote']["TradingPrice"]
+                    temp['Close']= realdata['Quote']["TradingPrice"]
+                    temp['Volume']= realdata['Quote']["TradeQuantity"]
+                    if "OpenInterest" in realdata['Quote'].keys():
+                        temp['OI']= realdata['Quote']["OpenInterest"]
+                    else:
+                        temp['OI']=""
+                    temp['QryIndex']= str(int(hisdata[-1]['QryIndex'])+1)
+                    hisdata.append(temp)
+                else:
+                    if  hisdata[-1]['High']<realdata['Quote']["TradingPrice"]:
+                        hisdata[-1]['High']=realdata['Quote']["TradingPrice"]
+                    elif hisdata[-1]['Low']>realdata['Quote']["TradingPrice"]:
+                        hisdata[-1]['Low']=realdata['Quote']["TradingPrice"]
+                    hisdata[-1]['Close']= realdata['Quote']["TradingPrice"]
+                    hisdata[-1]['Volume']= str(int(hisdata[-1]['Volume'])+int(realdata['Quote']["TradeQuantity"]))
+                    if "OpenInterest" in realdata['Quote'].keys():
+                        hisdata[-1]['OI']= realdata['Quote']["OpenInterest"]
+            elif bartype=="TICKS" :
+                temp['Symbol']=realdata['Quote']["Symbol"]
+                temp['DateTime']=realdata['Quote']["DateTime"]
+                temp['Date']=hisdata[-1]["Date"]                
+                temp['FilledTime']=realdata['Quote']['FilledTime']
+                temp['TradeQuantity']=realdata['Quote']['TradeQuantity']
+                temp['TradeVolume']=realdata['Quote']['TradeVolume']
+                temp['Bid']=realdata['Quote']['Bid']
+                temp['Ask']=realdata['Quote']['Ask']
+                temp['TradingPrice']=realdata['Quote']['TradingPrice']
+                temp['PreciseTime']=realdata['Quote']['PreciseTime'] 
+                temp['OI']=realdata['Quote']['OpenInterest']
+                temp['QryIndex']=str(int(hisdata[-1]['QryIndex'])+1)
+
+        return hisdata
+
+
+
+            
+
+
             
     #获取热门月历史对应的指定月数据
     #参数：
@@ -634,8 +830,8 @@ class QuoteClass():
         #print(sub_port)
         socket_sub.connect("tcp://127.0.0.1:%s" % sub_port)
         socket_sub.setsockopt_string(zmq.SUBSCRIBE,filter)
-        strQryIndex ={}
- 
+        #strQryIndex ={}
+
         while(threading.main_thread().isAlive()):
 
             message = (socket_sub.recv()[:-1]).decode("utf-8")
@@ -643,9 +839,16 @@ class QuoteClass():
             message = message[index:]
             message = json.loads(message)
             if(message["DataType"]=="TICKS" or message["DataType"]=="1K" or message["DataType"]=="DK" or message["DataType"]=="DOGSK"  or message["DataType"]=="DOGSS"):
-                self.event.set()
+                #del message['StartTime']
+                #del message['EndTime']
+                if 'DBType' in message.keys():
+                    del message['DBType']
+                if message not in self.status:
+                    self.status.append(message)
             elif message['DataType']!= 'PING':
-                self.eventQueue.put(message)             
+                self.eventQueue.put(message)
+                if message['DataType']=='REALTIME':
+                    self.realdate[message['Quote']['Symbol']]=message['Quote']['TradeDate']
         return
 
     #查询合约信息
@@ -678,6 +881,26 @@ class QuoteClass():
         self.m_objZMQKeepAlive = KeepAliveHelper(sessiondata,self,stkobj)
         
         return
+    class GethisThread(threading.Thread):
+        def __init__(self, func,symbol, type, startTime, endTime,dbtype):
+            threading.Thread.__init__(self)
+            self.funct=func
+            self.symb=symbol
+            self.bartype=type
+            self.startt=startTime
+            self.endt=endTime
+            self.db=dbtype
+            self.result=None
+
+        def run(self):
+            self.result =self.funct(self.symb,self.bartype,self.startt,self.endt,self.db)
+
+        def get_result(self):
+            threading.Thread.join(self) # 等待线程执行完毕
+            try:
+                return self.result
+            except Exception:
+                return None
 
 #心跳相关
 class KeepAliveHelper():
